﻿define([
    // Dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/_base/array",
    "dojo/dom-construct",
    "dojo/dom-class",
    "dojo/promise/all",
    "dojo/store/Memory",

    // Dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/_WidgetBase",
    "dijit/form/Button",
    //CMS
    "epi/dependency",
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/_ContentContextMixin",
    "epi/shell/widget/_ActionProviderWidget",
    // Episerver dnd classes
    "epi/shell/TypeDescriptorManager",
    "epi/shell/widget/dialog/Dialog",
    // dgrid
    "dgrid/OnDemandGrid",
    "dgrid/extensions/DnD",

    "bynder/editors/BynderMultiAssetSelectorDialog",
    "bynder/DerivativeHelper",

    "dojo/text!./BynderCompactViewComponent.html",
    "dojo/text!../Template/BynderError.html",
    "xstyle/css!./BynderCompactViewComponent.css"
], function(
    // Dojo
    declare,
    lang,
    array,
    domConstruct,
    domClass,
    all,
    Memory,

    // Dijit
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _WidgetBase,
    Button,
    //CMS
    dependency,
    _HasChildDialogMixin,
    _ContentContextMixin,
    _ActionProviderWidget,
    // Episerver dnd classes
    TypeDescriptorManager,
    Dialog,
    // dgrid
    OnDemandGrid,
    DnD,

    BynderMultiAssetSelectorDialog,
    DerivativeHelper,

    template,
    errorTemplate
) {
    return declare([_WidgetBase, _TemplatedMixin, _HasChildDialogMixin, _ContentContextMixin], {
        templateString: template,

        _memoryStore: null,

        button: null,
        _button: null,

        clearButton: null,
        _clearButton: null,

        source: null,
        _source: null,

        assetGrid: null,
        _assetGrid: null,

        compactViewOptions: null,

        postCreate: function() {
            this.inherited(arguments);

            this._memoryStore = new Memory({
                data: [],
                idProperty: "contentLink",
            });

            var registry = dependency.resolve("epi.storeregistry");
            this._store = registry.get("BynderAssetMappingStore");

            this._setupButtons();
            this._setupGrid();

            this.errorPanel.innerHTML = errorTemplate;
        },

        _setupButtons: function () {
            this._button = new Button({
                    label: "Select assets",
                    class: 'epi-flat epi-chromeless',
                    iconClass: "epi-iconPlus",
                    onClick: dojo.hitch(this, this._onButtonClick)
                },
                this.button);
            this._button.startup();

            this._clearButton = new Button({
                    label: "Clear",
                    class: 'epi-flat epi-chromeless',
                    iconClass: "epi-iconRevert",
                    onClick: dojo.hitch(this, this._onClearButtonClick)
                },
                this.clearButton);
            this._clearButton.startup();
        },

        _setupGrid: function() {
            this._assetGrid = new declare([OnDemandGrid, DnD])(
                {
                    class: "epi-bynder-grid",
                    store: this._memoryStore,
                    showHeader: false,
                    selectionMode: "single",
                    sort: "default",
                    dndParams: {
                        copyOnly: true,
                        selfAccept: false,
                        accept: [],
                        creator: this._createAvatarItem.bind(this),
                    },
                    getObjectDndType: this._getDndTypes.bind(this),
                    columns: [
                        {
                            field: "bynderImage",
                            label: "Image",
                            renderCell: function(item) {
                                return domConstruct.create("img", {
                                    src: item.properties.imageUrl,
                                    class: "epi-bynder-thumbnail",
                                });
                            },
                        },
                        {
                            field: "name",
                            label: "Name",
                        },
                    ],
                },
                this.assetGrid
            );

            this._assetGrid.refresh();
        },

        _onButtonClick: function() {
            this.contentSelectorDialog = new BynderMultiAssetSelectorDialog(
                this.compactViewOptions
            );

            this.dialog = new Dialog({
                title: "Select Bynder asset(s)",
                dialogClass: "epi-dialog-portrait",
                content: this.contentSelectorDialog,
                destroyOnHide: true,
            });

            this.dialog.own(this.contentSelectorDialog);

            this.connect(this.dialog, "onExecute", "_onDialogExecute");
            this.connect(this.dialog, "onHide", "_onDialogHide");
            this.connect(this.dialog, "onShow", "_onDialogShow");

            this.dialog.show();
        },

        _onClearButtonClick: function () {
            this._memoryStore.setData([]);
            this._assetGrid.refresh();
        },

        _onDialogExecute: function() {
            //summary:
            //    Handle dialog close
            // tags:
            //    private

            var assets = this.contentSelectorDialog.get("assets");
            array.forEach(assets, this._renderAsset, this);
        },

        _onDialogHide: function() {
            this.isShowingChildDialog = false;
        },

        _onDialogShow: function() {
            this.isShowingChildDialog = true;
        },

        _renderAsset: function(asset) {
            var derivatives = DerivativeHelper.getAvailableDerivatives(asset);
            var getContentPromises = array.map(
                derivatives,
                function(derivative) {
                    return this._store.query({
                        assetId: asset.id,
                        derivative: derivative,
                    });
                },
                this
            );

            all(getContentPromises).then(
                lang.hitch(this, function(contentList) {
                    this._updateValue(contentList);
                }),
                lang.hitch(this, function (error) {
                    domClass.add(this.bynderComponent, "epi-bynder--has-error");
                    console.error(error);
                })
            );
        },

        _updateValue: function(contentList) {
            array.forEach(
                contentList,
                function(contentItem) {
                    this._memoryStore.put(contentItem);
                },
                this
            );
            this._assetGrid.refresh();
        },

        _createAvatarItem: function(item) {
            //This node is used when dragging the item.
            return {
                node: domConstruct
                    .create("div")
                    .appendChild(domConstruct.create("img", {
                        src: item.properties.imageUrl,
                        class: "epi-bynder-thumbnail--large",
                    })),
                data: item,
            };
        },

        _getDndTypes: function(item) {
            return TypeDescriptorManager.getAndConcatenateValues(item.typeIdentifier, "dndTypes");
        },
    });
});
