﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/dom-class",
    "dojo/dom-prop",

    "dijit/_CssStateMixin",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",

    "epi-cms/widget/_Droppable",
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/widget/_HasClearButton",

    "epi/dependency",
    "epi/Url",
    "epi/routes",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/shell/widget/dialog/Dialog",

    "dojo/text!./BynderAssetSelector.html",

    "bynder/editors/BynderAssetSelectorDialog",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.thumbnailselector",
], function(
    declare,
    lang,
    when,
    domClass,
    domProp,

    _CssStateMixin,
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,

    _Droppable,
    _HasChildDialogMixin,
    _HasClearButton,

    dependency,
    Url,
    routes,
    _ValueRequiredMixin,
    Dialog,

    template,

    BynderAssetSelectorDialog,
    resources
) {
    return declare(
        "bynder.editors.BynderAssetSelector",
        [
            _Widget,
            _TemplatedMixin,
            _WidgetsInTemplateMixin,
            _HasChildDialogMixin,
            _HasClearButton,
            _ValueRequiredMixin,
            _Droppable,
        ],
        {
            templateString: template,
            resources: resources,
            intermediateChanges: false,

            compactViewOptions: null,
            allowedDerivatives: null,

            allowedDndTypes: ["bynder.contenttypes.bynderassetdata"],

            _contentRef: null,
            _store: null,

            postCreate: function() {
                // call base implementation
                this.inherited(arguments);

                this.selectedContentNameNode.innerText = this.value;
            },

            postMixInProperties: function() {
                // summary:
                //		Initialize properties
                // tags:
                //    protected

                this.inherited(arguments);

                var registry = dependency.resolve("epi.storeregistry");
                this._store = registry.get("BynderAssetStore");
            },

            destroy: function(/*Boolean*/ preserveDom) {
                // summary:
                //      Destroy this class, cancelling any promises that are in progress.
                // tags:
                //      public
                this.inherited(arguments);

                // If there is a value change promise in progress then cancel it.
                if (this._valueChangedPromise) {
                    this._valueChangedPromise.cancel();
                    this._valueChangedPromise = null;
                }
            },

            _setSelectedContentNameAttr: function(value) {
                // summary:
                //		Sets the display name for selected content
                // tags:
                //    protected

                // set the text
                domProp.set(this.selectedContentNameNode, "textContent", value);

                // set the title
                this._updateDisplayNodeTitle();
            },

            _updateDisplayNodeTitle: function() {
                // summary:
                //		updates the tooltip with the current text of displayNode
                // tags:
                //    protected

                this.resourceName.title = this.resourceName.textContent.trim();
            },

            _setValueAttr: function(value) {
                //summary:
                //    Value's setter.
                //
                // value: String
                //    Value to be set.
                //
                // tags:
                //    protected

                // Support drag and drop BynderAssetData
                if (typeof value === "object" && value && value.contentLink) {
                    value = value.contentLink;
                }

                this._contentRef = value;
                this._setValueAndFireOnChange(value);
            },

            _getValueAttr: function() {
                this.inherited(arguments);

                if (this._contentRef) {
                    return this._contentRef.toString();
                } else {
                    return "";
                }
            },

            _onButtonClick: function() {
                this.contentSelectorDialog = new BynderAssetSelectorDialog(
                    this.compactViewOptions,
                    this.allowedDerivatives
                );

                this.dialog = new Dialog({
                    title: "Select a Bynder asset",
                    dialogClass: "epi-dialog-portrait bynder-dialog",
                    content: this.contentSelectorDialog,
                    contentClass: "bynder-dialog-select-derivative",
                    destroyOnHide: true,
                });

                this.dialog.own(this.contentSelectorDialog);

                this.connect(this.dialog, "onExecute", "_onDialogExecute");
                this.connect(this.dialog, "onHide", "_onDialogHide");
                this.connect(this.dialog, "onShow", "_onDialogShow");

                this.dialog.show();
            },

            _onDialogExecute: function() {
                //summary:
                //    Handle dialog close
                // tags:
                //    private

                var value = this.contentSelectorDialog.get("value");
                this._setValueAndFireOnChange(value);
            },

            _setValueAndFireOnChange: function(value) {
                //summary:
                //    Gets the content data by selected value, updates the view and calls onChange if value was changed
                // tags:
                //    private

                var contentLink = value === "-" ? this.contentLink : value;

                this.set("permanentLink", null);
                this.set("previewUrl", null);

                this._valueChangedPromise = when(
                    this._getContentData(contentLink),
                    lang.hitch(this, function(content) {
                        // Clear the pointer to the promise since it is resolved.
                        this._valueChangedPromise = null;

                        var hasChange = this.value !== contentLink;

                        this.value = contentLink;
                        this._started && this.validate();
                        this._updateDisplayNode(content);

                        if (hasChange) {
                            this.selectedContentNameNode.innerText = contentLink;
                            this.set("value", contentLink);
                            this.onChange(contentLink);
                        }

                        if (content) {
                            this.set("permanentLink", content.permanentLink);
                            this.set("previewUrl", content.previewUrl);
                        }
                    })
                );
            },

            _getThumbnailUrl: function(content) {
                var url = new Url(
                    routes.getActionPath({
                        moduleArea: "CMS",
                        controller: "Thumbnail",
                        action: "Generate",
                    })
                );
                url.query = {
                    contentLink: content.contentLink,
                    "epi.preventCache": new Date().valueOf(),
                };
                return url;
            },

            _updateDisplayNode: function(content) {
                //summary:
                //    Update widget's display text and title.
                // tags:
                //    protected
                var name = content ? content.name : "";
                if (content) {
                    domClass.remove(this.domNode, "epi-noValue");
                    domClass.remove(this.resourceName, "dijitPlaceHolder");
                    this.set("selectedContentName", name);
                } else {
                    domClass.add(this.domNode, "epi-noValue");
                    domClass.add(this.resourceName, "dijitPlaceHolder");
                    this.set("selectedContentName", "");
                }

                // From ThumbnailSelector
                if (content) {
                    this.thumbnail.src = content.properties.imageUrl;
                }
                this.stateNode.title = name;

                // Toggle preview
                domClass.toggle(this.displayNode, "dijitHidden", !content);
                domClass.toggle(this.actionsContainer, "dijitHidden", content);
            },

            _getContentData: function(contentLink) {
                //summary:
                //    Loads the content data from the store
                // tags:
                //    private

                if (!contentLink) {
                    return null;
                }
                return this._store.get(contentLink);
            },

            _onDialogHide: function() {
                this.isShowingChildDialog = false;

                // Return focus to the editor when the dialog closes.
                this.focus();
            },

            _onDialogShow: function() {
                //summary:
                //    Handle onShow dialog event.
                // tags:
                //    protected

                this.isShowingChildDialog = true;
            },
        }
    );
});
