﻿define([
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dom-construct",
    "dojo/dom-class",

    "dijit/_CssStateMixin",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

    "epi/dependency",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/shell/widget/_ActionProviderWidget",

    "bynder/DerivativeHelper",

    "dojo/text!./BynderAssetSelectorDialog.html",
    "dojo/text!../Template/BynderError.html",

    "dijit/MenuItem",
    "xstyle/css!./BynderAssetSelectorDialog.css",
], function(
    declare,
    lang,
    domConstruct,
    domClass,

    _CssStateMixin,
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

    dependency,
    _ValueRequiredMixin,
    _ActionProviderWidget,

    DerivativeHelper,

    template,
    errorTemplate,

    MenuItem,
) {
    return declare(
        "bynder.editors.BynderAssetSelectorDialog",
        [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionProviderWidget],
        {
            templateString: template,
            intermediateChanges: false,
            _asset: null,
            _store: null,
            _script: null,
            _eventListener: null,

            _compactViewOptions: null,
            _allowedDerivatives: null,

            constructor: function(compactViewOptions, allowedDerivatives) {
                this._compactViewOptions = compactViewOptions;
                this._allowedDerivatives = allowedDerivatives;
            },

            postCreate: function() {
                // call base implementation
                this.inherited(arguments);

                var registry = dependency.resolve("epi.storeregistry");
                this._store = registry.get("BynderAssetMappingStore");

                // Set up event listener
                this._eventListener = lang.hitch(this, this._bynderAddMedia);
                document.addEventListener("BynderAddMedia", this._eventListener);

                this._loadCompactView();

                this.errorPanel.innerHTML = errorTemplate;
            },

            destroy: function() {
                // summary:
                //      Destroy the widget and hide the dialog.
                // tags:
                //      protected

                this.inherited(arguments);

                document.removeEventListener("BynderAddMedia", this._eventListener);
                document.head.removeChild(this._script);
            },

            _loadCompactView: function () {
                // Create div
                this.compactView = domConstruct.create('div',
                    {
                        id: 'bynder-compactview'
                    });
                // Init compact view options
                if (this._compactViewOptions) {
                    for (var option in this._compactViewOptions) {
                        this.compactView.setAttribute(
                            "data-" + option,
                            this._compactViewOptions[option]
                        );
                        // Force single asset selection mode
                        this.compactView.setAttribute(
                            "data-mode",
                            "single"
                        );
                    }
                }
                // Place div in DOM
                domConstruct.place(this.compactView, this.assetContainer, 'before');

                // Load bynder CV script
                this._script = document.createElement("script");
                this._script.src =
                    "https://d8ejoa1fys2rk.cloudfront.net/modules/compactview/includes/js/client-1.5.0.min.js";
                document.head.appendChild(this._script);
            },

            _bynderAddMedia: function(e) {
                // Close CV after selection
                this.compactView.parentNode.removeChild(this.compactView);
                this.compactView = null;

                // The selected assets are found in the event detail property
                var selectedAssets = e.detail;

                this.assetContainer.innerHTML = "<h2>Loading...</h2>";

                var asset = selectedAssets[0];
                var derivatives = DerivativeHelper.getAvailableDerivatives(
                    asset,
                    this._allowedDerivatives
                );
                var selectedDerivative = derivatives[0];
                this.derivativeDropDownButton.set("label", selectedDerivative);
                this._buildMetaDataGrid(asset, selectedDerivative);

                this._store.query({ assetId: asset.id, derivative: selectedDerivative })
                    .then(function (data) {
                            this._renderAsset(data, asset, derivatives);
                        }.bind(this),
                        function (error) {
                            this._renderError(error);
                        }.bind(this));
            },

            _buildMetaDataGrid: function (asset, selectedDerivative) {
                this.derivativeMetaDataGrid.refresh();
                const gridData = [
                    { key: "Orientation", value: asset.orientation },
                    { key: "Type", value: asset.type },
                    { key: "File size", value: asset.fileSize },
                    { key: "Name", value: asset.name },
                    { key: "Copyright", value: asset.copyright },
                    { key: "Extension", value: asset.extension },
                    { key: "Date published", value: asset.datePublished },
                    { key: "Date created", value: asset.dateCreated },
                    { key: "Date modified", value: asset.dateModified },
                ];
                this.derivativeMetaDataGrid.renderArray(gridData);
            },

            _onSelectDerivative: function(asset, evt) {
                const derivative = evt.srcElement.innerText;
                this.derivativeDropDownButton.set("label", derivative);
                dojo.when(
                    this._store.query({ assetId: asset.id, derivative: derivative }),
                    lang.hitch(this, function(data) {
                        this._updateValue(data.contentLink, asset);
                    })
                );
            },

            _renderAsset: function(data, asset, derivatives) {
                this.assetContainer.innerHTML = "";

                this._updateValue(data.contentLink, asset);

                switch (asset.type) {
                case "audio":
                case "document":
                case "image":
                    this.assetContainer.innerHTML +=
                        "<img src='" +
                        asset.thumbnails["thul"] +
                        "' title='" +
                        asset.name +
                        "' />";

                    for (var thumbnail in asset.thumbnails) {
                        if (derivatives.indexOf(thumbnail) !== -1) {
                            this.derivativeDropDownMenu.addChild(
                                new MenuItem({
                                    label: thumbnail,
                                    onClick: lang.hitch(this,
                                        function (evt) {
                                            this._onSelectDerivative(asset, evt);
                                        }),
                                })
                            );
                        }
                    }
                    break;
                case "video":
                    this.assetContainer.innerHTML +=
                        '<video height="150" controls>' +
                        '<source src="' +
                        asset.videoPreviewURLs[0] +
                        '" type="video/mp4">' +
                        "</video>";
                    break;
                }
            },

            _renderError: function (error) {
                domClass.add(this.inputContainer, "epi-bynder--has-error");
                console.error(error);
            },

            _updateValue: function(value, asset) {
                this.set("value", value);
                this.set("asset", asset);
            },

            _onRemoveButtonClick: function () {
                this._updateValue(null, null);
                this.derivativeDropDownMenu.destroyDescendants();
                this._loadCompactView();
            },

            focus: function() {},
        }
    );
});
