﻿define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/_base/lang",

    "dijit/_CssStateMixin",
    "dijit/_Widget",
    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

    "epi/dependency",
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/shell/widget/_ActionProviderWidget",

    "dojo/text!./BynderMultiAssetSelectorDialog.html"
], function(
    array,
    declare,
    lang,

    _CssStateMixin,
    _Widget,
    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

    dependency,
    _ValueRequiredMixin,
    _ActionProviderWidget,

    template
) {
    return declare(
        "bynder.editors.BynderMultiAssetSelectorDialog",
        [_Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _ActionProviderWidget],
        {
            templateString: template,
            intermediateChanges: false,

            compactView: null,
            assetContainer: null,

            _store: null,
            _script: null,
            _eventListener: null,

            _compactViewOptions: null,

            constructor: function(compactViewOptions) {
                this._compactViewOptions = compactViewOptions;
            },

            postCreate: function() {
                // call base implementation
                this.inherited(arguments);

                var registry = dependency.resolve("epi.storeregistry");
                this._store = registry.get("BynderAssetMappingStore");

                // Init compact view options
                if (this._compactViewOptions) {
                    for (var option in this._compactViewOptions) {
                        this.compactView.setAttribute(
                            "data-" + option,
                            this._compactViewOptions[option]
                        );
                    }
                    // Force multi asset selection mode
                    this.compactView.setAttribute(
                        "data-mode",
                        "multi"
                    );
                }

                // Set up event listener
                this._eventListener = lang.hitch(this, this._bynderAddMedia);
                document.addEventListener("BynderAddMedia", this._eventListener);

                // Load bynder CV script
                this._script = document.createElement("script");
                this._script.src =
                    "https://d8ejoa1fys2rk.cloudfront.net/modules/compactview/includes/js/client-1.5.0.min.js";
                document.head.appendChild(this._script);
            },

            destroy: function() {
                // summary:
                //      Destroy the widget and hide the dialog.
                // tags:
                //      protected

                this.inherited(arguments);

                document.removeEventListener("BynderAddMedia", this._eventListener);
                document.head.removeChild(this._script);
            },

            _bynderAddMedia: function(e) {
                // Close CV after selection
                var compactView = document.getElementById("bynder-compactview");
                compactView.parentNode.removeChild(compactView);

                this.assetContainer.innerHTML = "<h2>Loading...</h2>";

                // The selected assets are found in the event detail property
                var selectedAssets = e.detail;
                this._updateValue(selectedAssets);
                
                this.assetContainer.innerHTML = "<h2>" + selectedAssets.length + " assets selected:</h2>";
                array.forEach(selectedAssets, this._renderAsset, this);
            },

            _renderAsset: function(asset) {
                switch (asset.type) {
                case "audio":
                case "document":
                case "image":
                    this.assetContainer.innerHTML +=
                        "<div><img src='" +
                        asset.thumbnails["thul"] +
                        "' title='" +
                        asset.name +
                        "' /></div><span>" + asset.name + "</span><hr />";
                    break;
                case "video":
                    this.assetContainer.innerHTML +=
                        '<div><video width="150" controls>' +
                        '<source src="' +
                        asset.videoPreviewURLs[0] +
                        '" type="video/mp4">' +
                        "</video></div><span>" + asset.name + "</span><hr />";
                    break;
                }
            },

            _updateValue: function(assets) {
                this.set("assets", assets);
            },

            _onRemoveButtonClick: function() {},

            focus: function() {},
        }
    );
});
